from dagster import check
from dagster.core.definitions.events import AssetKey

from .asset import AssetsDefinition
from .partition_key_range import PartitionKeyRange


def get_parent_partitions_for_partition_range(
    child_assets_def: AssetsDefinition,
    parent_assets_def: AssetsDefinition,
    parent_asset_key: AssetKey,
    child_partition_key_range: PartitionKeyRange,
) -> PartitionKeyRange:
    """Returns the range of partition keys in the parent asset that include data necessary
    to compute the contents of the given partition key range in the child asset.
    """

    if child_assets_def.partitions_def is None:
        check.failed("Child asset is not partitioned")

    if parent_assets_def.partitions_def is None:
        check.failed("Parent asset is not partitioned")

    child_partition_mapping = child_assets_def.get_partition_mapping(parent_asset_key)
    return child_partition_mapping.get_parent_partitions_for_partition_range(
        child_assets_def.partitions_def, parent_assets_def.partitions_def, child_partition_key_range
    )


def get_child_partitions_for_partition_range(
    child_assets_def: AssetsDefinition,
    parent_assets_def: AssetsDefinition,
    parent_asset_key: AssetKey,
    parent_partition_key_range: PartitionKeyRange,
) -> PartitionKeyRange:
    """Returns the range of partition keys in the child asset that use the data in the given
    partition key range of the parent asset.
    """
    if child_assets_def.partitions_def is None:
        check.failed("Child asset is not partitioned")

    if parent_assets_def.partitions_def is None:
        check.failed("Parent asset is not partitioned")

    child_partition_mapping = child_assets_def.get_partition_mapping(parent_asset_key)
    return child_partition_mapping.get_child_partitions_for_partition_range(
        child_assets_def.partitions_def,
        parent_assets_def.partitions_def,
        parent_partition_key_range,
    )



import string

from dagster import (
    InputDefinition,
    Int,
    OutputDefinition,
    PartitionSetDefinition,
    ScheduleDefinition,
    lambda_solid,
    pipeline,
    repository,
    solid,
    usable_as_dagster_type,
)
from dagster.core.definitions.decorators.sensor import sensor
from dagster.core.definitions.sensor_definition import RunRequest
from dagster.core.test_utils import default_mode_def_for_test


@lambda_solid
def do_something():
    return 1


@lambda_solid
def do_input(x):
    return x


@pipeline(name="foo", mode_defs=[default_mode_def_for_test])
def foo_pipeline():
    do_input(do_something())


@pipeline(name="baz", description="Not much tbh")
def baz_pipeline():
    do_input()


def define_foo_pipeline():
    return foo_pipeline


@pipeline(name="other_foo")
def other_foo_pipeline():
    do_input(do_something())


def define_other_foo_pipeline():
    return other_foo_pipeline


@pipeline(name="bar")
def bar_pipeline():
    @usable_as_dagster_type(name="InputTypeWithoutHydration")
    class InputTypeWithoutHydration(int):
        pass

    @solid(output_defs=[OutputDefinition(InputTypeWithoutHydration)])
    def one(_):
        return 1

    @solid(
        input_defs=[InputDefinition("some_input", InputTypeWithoutHydration)],
        output_defs=[OutputDefinition(Int)],
    )
    def fail_subset(_, some_input):
        return some_input

    return fail_subset(one())


def define_bar_schedules():
    return {
        "foo_schedule": ScheduleDefinition(
            "foo_schedule",
            cron_schedule="* * * * *",
            pipeline_name="foo",
            run_config={"fizz": "buzz"},
        ),
        "foo_schedule_never_execute": ScheduleDefinition(
            "foo_schedule_never_execute",
            cron_schedule="* * * * *",
            pipeline_name="foo",
            run_config={"fizz": "buzz"},
            should_execute=lambda _context: False,
        ),
        "foo_schedule_echo_time": ScheduleDefinition(
            "foo_schedule_echo_time",
            cron_schedule="* * * * *",
            pipeline_name="foo",
            run_config_fn=lambda context: {
                "passed_in_time": context.scheduled_execution_time.isoformat()
                if context.scheduled_execution_time
                else ""
            },
        ),
    }


def error_partition_fn():
    raise Exception("womp womp")


def error_partition_config_fn():
    raise Exception("womp womp")


def error_partition_tags_fn(_partition):
    raise Exception("womp womp")


def define_baz_partitions():
    return {
        "baz_partitions": PartitionSetDefinition(
            name="baz_partitions",
            pipeline_name="baz",
            partition_fn=lambda: string.ascii_lowercase,
            run_config_fn_for_partition=lambda partition: {
                "solids": {"do_input": {"inputs": {"x": {"value": partition.value}}}}
            },
            tags_fn_for_partition=lambda _partition: {"foo": "bar"},
        ),
        "error_partitions": PartitionSetDefinition(
            name="error_partitions",
            pipeline_name="baz",
            partition_fn=error_partition_fn,
            run_config_fn_for_partition=lambda partition: {},
        ),
        "error_partition_config": PartitionSetDefinition(
            name="error_partition_config",
            pipeline_name="baz",
            partition_fn=lambda: string.ascii_lowercase,
            run_config_fn_for_partition=error_partition_config_fn,
        ),
        "error_partition_tags": PartitionSetDefinition(
            name="error_partition_tags",
            pipeline_name="baz",
            partition_fn=lambda: string.ascii_lowercase,
            run_config_fn_for_partition=lambda partition: {},
            tags_fn_for_partition=error_partition_tags_fn,
        ),
    }


@sensor(pipeline_name="foo")
def sensor_foo(_):
    yield RunRequest(run_key=None, run_config={"foo": "FOO"}, tags={"foo": "foo_tag"})
    yield RunRequest(run_key=None, run_config={"foo": "FOO"})


@sensor(pipeline_name="foo")
def sensor_error(_):
    raise Exception("womp womp")


@repository
def bar_repo():
    return {
        "pipelines": {
            "foo": define_foo_pipeline,
            "bar": lambda: bar_pipeline,
            "baz": lambda: baz_pipeline,
        },
        "schedules": define_bar_schedules(),
        "partition_sets": define_baz_partitions(),
        "sensors": {"sensor_foo": sensor_foo, "sensor_error": lambda: sensor_error},
    }


@repository
def other_repo():
    return {"pipelines": {"other_foo": define_other_foo_pipeline}}



from abc import ABC, abstractmethod

from dagster.core.definitions.partition import PartitionsDefinition

from .partition_key_range import PartitionKeyRange


class PartitionMapping(ABC):
    """Defines a correspondence between the partitions in an asset and the partitions in an asset
    that it depends on.
    """

    @abstractmethod
    def get_parent_partitions_for_partition_range(
        self,
        child_partitions_def: PartitionsDefinition,
        parent_partitions_def: PartitionsDefinition,
        child_partition_key_range: PartitionKeyRange,
    ) -> PartitionKeyRange:
        """Returns the range of partition keys in the parent asset that include data necessary
        to compute the contents of the given partition key range in the child asset.

        Args:
            child_partitions_def (PartitionsDefinition): The partitions definition for the child
                asset.
            parent_partitions_def (PartitionsDefinition): The partitions definition for the parent
                asset.
            child_partition_key_range (PartitionKeyRange): The range of partition keys in the child
                asset.
        """

    @abstractmethod
    def get_child_partitions_for_partition_range(
        self,
        child_partitions_def: PartitionsDefinition,
        parent_partitions_def: PartitionsDefinition,
        parent_partition_key_range: PartitionKeyRange,
    ) -> PartitionKeyRange:
        """Returns the range of partition keys in the child asset that use the data in the given
        partition key range of the child asset.

        Args:
            child_partitions_def (PartitionsDefinition): The partitions definition for the child
                asset.
            parent_partitions_def (PartitionsDefinition): The partitions definition for the parent
                asset.
            parent_partition_key_range (PartitionKeyRange): The range of partition keys in the
                parent asset.
        """


class IdentityPartitionMapping(PartitionMapping):
    def get_parent_partitions_for_partition_range(
        self,
        child_partitions_def: PartitionsDefinition,  # pylint: disable=unused-argument
        parent_partitions_def: PartitionsDefinition,  # pylint: disable=unused-argument
        child_partition_key_range: PartitionKeyRange,
    ) -> PartitionKeyRange:
        return child_partition_key_range

    def get_child_partitions_for_partition_range(
        self,
        child_partitions_def: PartitionsDefinition,  # pylint: disable=unused-argument
        parent_partitions_def: PartitionsDefinition,  # pylint: disable=unused-argument
        parent_partition_key_range: PartitionKeyRange,
    ) -> PartitionKeyRange:
        return parent_partition_key_range



from dagster import check
from dagster.core.errors import DagsterInvariantViolationError, DagsterRunNotFoundError
from dagster.core.execution.context.system import IPlanContext
from dagster.core.execution.plan.plan import ExecutionPlan


def validate_reexecution_memoization(
    plan_context: IPlanContext, execution_plan: ExecutionPlan
) -> None:

    parent_run_id = plan_context.pipeline_run.parent_run_id
    check.opt_str_param(parent_run_id, "parent_run_id")

    if parent_run_id is None:
        return

    if not plan_context.instance.has_run(parent_run_id):
        raise DagsterRunNotFoundError(
            "Run id {} set as parent run id was not found in instance".format(parent_run_id),
            invalid_run_id=parent_run_id,
        )

    # exclude full pipeline re-execution
    if len(execution_plan.step_keys_to_execute) == len(execution_plan.steps):
        return

    if execution_plan.artifacts_persisted:
        return

    raise DagsterInvariantViolationError(
        "Cannot perform reexecution with in-memory io managers.\n"
        "To enable reexecution, you can set a persistent io manager, such as the "
        'fs_io_manager, in the resource_defs argument on your job: resource_defs={"io_manager": fs_io_manager}'
    )



from .composite_solid import composite_solid
from .config_mapping import config_mapping
from .graph import graph
from .hook import failure_hook, success_hook
from .job import job
from .op import op
from .pipeline import pipeline
from .repository import repository
from .schedule import daily_schedule, hourly_schedule, monthly_schedule, schedule, weekly_schedule
from .sensor import asset_sensor, sensor
from .solid import lambda_solid, solid
